@@ -56,9 +56,12 @@ import org.xml.sax.SAXParseException;
 
 import ui.DOMTreeFull;
 
-/** This class shows a DOM Document in a JTree, and presents controls
- *  which allow the user to create and view the progress of a TreeWalker
- *  in the DOM tree.
+/** 
+ * This class shows a DOM Document in a JTree, and presents controls
+ * which allow the user to create and view the progress of a TreeWalker
+ * in the DOM tree.
+ * 
+ * @version $Id: TreeWalkerView.java 950355 2010-06-02 03:43:24Z mrglavas $
  */
 public class TreeWalkerView 
     extends JFrame 
@@ -400,7 +403,7 @@ public class TreeWalkerView
             if (nameNodeFilter==null) {
                 nameNodeFilter = new NameNodeFilter();
             }
-            if (nameText.equals("")) {
+            if (nameText.length() == 0) {
                 setMessage("NodeNameFilter name is \"\". Assuming null.");
                 nameText = null;
             }
