@@ -1,9 +1,10 @@
 /*
- * Copyright 1999-2006 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -16,15 +17,11 @@
 
 package sax;
 
-import java.lang.reflect.Method;
-
 import java.io.OutputStream;
 import java.io.OutputStreamWriter;
 import java.io.PrintWriter;
 import java.io.UnsupportedEncodingException;
 
-import sax.helpers.AttributesImpl;
-
 import org.xml.sax.Attributes;
 import org.xml.sax.Locator;
 import org.xml.sax.Parser;
@@ -34,11 +31,14 @@ import org.xml.sax.SAXNotSupportedException;
 import org.xml.sax.SAXParseException;
 import org.xml.sax.XMLReader;
 import org.xml.sax.ext.LexicalHandler;
+import org.xml.sax.ext.Locator2;
 import org.xml.sax.helpers.DefaultHandler;
 import org.xml.sax.helpers.ParserAdapter;
 import org.xml.sax.helpers.ParserFactory;
 import org.xml.sax.helpers.XMLReaderFactory;
 
+import sax.helpers.AttributesImpl;
+
 /**
  * A sample SAX2 writer. This sample program illustrates how to
  * register a SAX2 ContentHandler and receive the callbacks in
@@ -46,7 +46,7 @@ import org.xml.sax.helpers.XMLReaderFactory;
  *
  * @author Andy Clark, IBM
  *
- * @version $Id: Writer.java 429930 2006-08-09 02:33:11Z mrglavas $
+ * @version $Id: Writer.java 637425 2008-03-15 15:10:11Z mrglavas $
  */
 public class Writer
     extends DefaultHandler
@@ -250,21 +250,28 @@ public class Writer
 
         // Root Element
         if (fElementDepth == 0) {
+            String encoding = "UTF-8";
             if (fLocator != null) {
-                fXML11 = "1.1".equals(getVersion());
+                if (fLocator instanceof Locator2) {
+                    Locator2 locator2 = (Locator2) fLocator;
+                    fXML11 = "1.1".equals(locator2.getXMLVersion());
+                    encoding = locator2.getEncoding();
+                    if (encoding == null) {
+                        encoding = "UTF-8";
+                    }
+                }
                 fLocator = null;
             }
 
             // The XML declaration cannot be printed in startDocument because
-            // the version reported by the Locator cannot be relied on until after
-            // the XML declaration in the instance document has been read.
+            // the version and encoding information reported by the Locator 
+            // cannot be relied on until the next event after startDocument.
             if (!fCanonical) {
-                if (fXML11) {
-                    fOut.println("<?xml version=\"1.1\" encoding=\"UTF-8\"?>");
-                }
-                else {
-                    fOut.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
-                }
+                fOut.print("<?xml version=\"");
+                fOut.print(fXML11 ? "1.1" : "1.0");
+                fOut.print("\" encoding=\"");
+                fOut.print(encoding);
+                fOut.println("\"?>");
                 fOut.flush();
             }
         }
@@ -529,27 +536,6 @@ public class Writer
 
     } // printError(String,SAXParseException)
 
-    /** Extracts the XML version from the Locator. */
-    protected String getVersion() {
-        if (fLocator == null) {
-            return null;
-        }
-        String version = null;
-        Method getXMLVersion = null;
-        try {
-            getXMLVersion = fLocator.getClass().getMethod("getXMLVersion", new Class[]{});
-            // If Locator implements Locator2, this method will exist.
-            if (getXMLVersion != null) {
-                version = (String) getXMLVersion.invoke(fLocator, (Object[]) null);
-            }
-        } 
-        catch (Exception e) { 
-            // Either this locator object doesn't have 
-            // this method, or we're on an old JDK.
-        }
-        return version;
-    } // getVersion()
-
     //
     // Main
     //
