@@ -17,14 +17,13 @@
 
 package xni;
 
-import java.io.InputStream;
-import java.io.IOException;
+import java.io.BufferedReader;
 import java.io.File;
 import java.io.FileInputStream;
-
-import java.util.Properties;
-import java.io.BufferedReader;
+import java.io.IOException;
+import java.io.InputStream;
 import java.io.InputStreamReader;
+import java.util.Properties;
 
 /**
  * This class is duplicated for each JAXP subpackage so keep it in sync.
@@ -41,7 +40,7 @@ import java.io.InputStreamReader;
  * class and modified to be used as a general utility for creating objects
  * dynamically.
  *
- * @version $Id: ObjectFactory.java 558590 2007-07-23 02:43:11Z mrglavas $
+ * @version $Id: ObjectFactory.java 924308 2010-03-17 14:36:25Z mrglavas $
  */
 final class ObjectFactory {
 
@@ -53,7 +52,7 @@ final class ObjectFactory {
     private static final String DEFAULT_PROPERTIES_FILENAME = "xerces.properties";
 
     /** Set to true for debugging */
-    private static final boolean DEBUG = false;
+    private static final boolean DEBUG = isDebugEnabled();
     
     /**
      * Default columns per line.
@@ -135,7 +134,7 @@ final class ObjectFactory {
         // Use the system property first
         try {
             String systemProp = SecuritySupport.getSystemProperty(factoryId);
-            if (systemProp != null) {
+            if (systemProp != null && systemProp.length() > 0) {
                 if (DEBUG) debugPrintln("found system property, value=" + systemProp);
                 return newInstance(systemProp, cl, true);
             }
@@ -257,11 +256,22 @@ final class ObjectFactory {
     //
     // Private static methods
     //
+    
+    /** Returns true if debug has been enabled. */
+    private static boolean isDebugEnabled() {
+        try {
+            String val = SecuritySupport.getSystemProperty("xerces.debug");
+            // Allow simply setting the prop to turn on debug
+            return (val != null && (!"false".equals(val)));
+        } 
+        catch (SecurityException se) {}
+        return false;
+    } // isDebugEnabled()
 
     /** Prints a message to standard error if debugging is enabled. */
     private static void debugPrintln(String msg) {
         if (DEBUG) {
-            System.err.println("JAXP: " + msg);
+            System.err.println("XERCES: " + msg);
         }
     } // debugPrintln(String)
 
