@@ -1,9 +1,10 @@
 /*
- * Copyright 1999-2005 The Apache Software Foundation.
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -82,7 +83,7 @@ import org.xml.sax.SAXNotSupportedException;
  *
  * @author Arun  Yadav,Sun Miscrosystem.
  * @author Peter McCracken, IBM
- * @version $Id: PSVIWriter.java 320203 2005-01-11 13:47:23Z mrglavas $
+ * @version $Id: PSVIWriter.java 778248 2009-05-24 22:39:36Z mrglavas $
  */
 public class PSVIWriter implements XMLComponent, XMLDocumentFilter {
 
@@ -1981,7 +1982,7 @@ public class PSVIWriter implements XMLComponent, XMLDocumentFilter {
         String elementName,
         Vector attributes,
         XMLString elementValue) {
-        if (elementValue == null || elementValue.equals("")) {
+        if (elementValue == null || elementValue.length == 0) {
             if (attributes == null) {
                 attributes = new Vector();
             }
