@@ -17,14 +17,13 @@
 
 package org.apache.html.dom;
 
-import java.io.InputStream;
-import java.io.IOException;
+import java.io.BufferedReader;
 import java.io.File;
 import java.io.FileInputStream;
-
-import java.util.Properties;
-import java.io.BufferedReader;
+import java.io.IOException;
+import java.io.InputStream;
 import java.io.InputStreamReader;
+import java.util.Properties;
 
 /**
  * This class is duplicated for each JAXP subpackage so keep it in sync.
@@ -40,7 +39,7 @@ import java.io.InputStreamReader;
  * 
  * @xerces.internal
  *
- * @version $Id: ObjectFactory.java 447255 2006-09-18 05:36:42Z mrglavas $
+ * @version $Id: ObjectFactory.java 924307 2010-03-17 14:36:05Z mrglavas $
  */
 final class ObjectFactory {
 
@@ -52,7 +51,7 @@ final class ObjectFactory {
     private static final String DEFAULT_PROPERTIES_FILENAME = "xerces.properties";
 
     /** Set to true for debugging */
-    private static final boolean DEBUG = false;
+    private static final boolean DEBUG = isDebugEnabled();
     
     /**
      * Default columns per line.
@@ -129,13 +128,12 @@ final class ObjectFactory {
     {
         if (DEBUG) debugPrintln("debug is on");
 
-        SecuritySupport ss = SecuritySupport.getInstance();
         ClassLoader cl = findClassLoader();
 
         // Use the system property first
         try {
-            String systemProp = ss.getSystemProperty(factoryId);
-            if (systemProp != null) {
+            String systemProp = SecuritySupport.getSystemProperty(factoryId);
+            if (systemProp != null && systemProp.length() > 0) {
                 if (DEBUG) debugPrintln("found system property, value=" + systemProp);
                 return newInstance(systemProp, cl, true);
             }
@@ -150,11 +148,11 @@ final class ObjectFactory {
             File propertiesFile = null;
             boolean propertiesFileExists = false;
             try {
-                String javah = ss.getSystemProperty("java.home");
+                String javah = SecuritySupport.getSystemProperty("java.home");
                 propertiesFilename = javah + File.separator +
                     "lib" + File.separator + DEFAULT_PROPERTIES_FILENAME;
                 propertiesFile = new File(propertiesFilename);
-                propertiesFileExists = ss.getFileExists(propertiesFile);
+                propertiesFileExists = SecuritySupport.getFileExists(propertiesFile);
             } catch (SecurityException e) {
                 // try again...
                 fLastModified = -1;
@@ -168,7 +166,7 @@ final class ObjectFactory {
                     // file existed last time
                     if(fLastModified >= 0) {
                         if(propertiesFileExists &&
-                                (fLastModified < (fLastModified = ss.getLastModified(propertiesFile)))) {
+                                (fLastModified < (fLastModified = SecuritySupport.getLastModified(propertiesFile)))) {
                             loadProperties = true;
                         } else {
                             // file has stopped existing...
@@ -181,13 +179,13 @@ final class ObjectFactory {
                         // file has started to exist:
                         if(propertiesFileExists) {
                             loadProperties = true;
-                            fLastModified = ss.getLastModified(propertiesFile);
+                            fLastModified = SecuritySupport.getLastModified(propertiesFile);
                         } // else, nothing's changed
                     }
                     if(loadProperties) {
                         // must never have attempted to read xerces.properties before (or it's outdeated)
                         fXercesProperties = new Properties();
-                        fis = ss.getFileInputStream(propertiesFile);
+                        fis = SecuritySupport.getFileInputStream(propertiesFile);
                         fXercesProperties.load(fis);
                     }
                 } catch (Exception x) {
@@ -214,7 +212,7 @@ final class ObjectFactory {
         } else {
             FileInputStream fis = null;
             try {
-                fis = ss.getFileInputStream(new File(propertiesFilename));
+                fis = SecuritySupport.getFileInputStream(new File(propertiesFilename));
                 Properties props = new Properties();
                 props.load(fis);
                 factoryClassName = props.getProperty(factoryId);
@@ -257,11 +255,22 @@ final class ObjectFactory {
     //
     // Private static methods
     //
+    
+    /** Returns true if debug has been enabled. */
+    private static boolean isDebugEnabled() {
+        try {
+            String val = SecuritySupport.getSystemProperty("xerces.debug");
+            // Allow simply setting the prop to turn on debug
+            return (val != null && (!"false".equals(val)));
+        } 
+        catch (SecurityException se) {}
+        return false;
+    } // isDebugEnabled()
 
     /** Prints a message to standard error if debugging is enabled. */
     private static void debugPrintln(String msg) {
         if (DEBUG) {
-            System.err.println("JAXP: " + msg);
+            System.err.println("XERCES: " + msg);
         }
     } // debugPrintln(String)
 
@@ -272,12 +281,10 @@ final class ObjectFactory {
     static ClassLoader findClassLoader()
         throws ConfigurationError
     {
-        SecuritySupport ss = SecuritySupport.getInstance();
-
         // Figure out which ClassLoader to use for loading the provider
         // class.  If there is a Context ClassLoader then use it.
-        ClassLoader context = ss.getContextClassLoader();
-        ClassLoader system = ss.getSystemClassLoader();
+        ClassLoader context = SecuritySupport.getContextClassLoader();
+        ClassLoader system = SecuritySupport.getSystemClassLoader();
 
         ClassLoader chain = system;
         while (true) {
@@ -302,7 +309,7 @@ final class ObjectFactory {
                     if (chain == null) {
                         break;
                     }
-                    chain = ss.getParentClassLoader(chain);
+                    chain = SecuritySupport.getParentClassLoader(chain);
                 }
 
                 // Assert: Current ClassLoader not in chain of
@@ -317,7 +324,7 @@ final class ObjectFactory {
 
             // Check for any extension ClassLoaders in chain up to
             // boot ClassLoader
-            chain = ss.getParentClassLoader(chain);
+            chain = SecuritySupport.getParentClassLoader(chain);
         };
 
         // Assert: Context ClassLoader not in chain of
@@ -409,21 +416,20 @@ final class ObjectFactory {
     private static Object findJarServiceProvider(String factoryId)
         throws ConfigurationError
     {
-        SecuritySupport ss = SecuritySupport.getInstance();
         String serviceId = "META-INF/services/" + factoryId;
         InputStream is = null;
 
         // First try the Context ClassLoader
         ClassLoader cl = findClassLoader();
 
-        is = ss.getResourceAsStream(cl, serviceId);
+        is = SecuritySupport.getResourceAsStream(cl, serviceId);
 
         // If no provider found then try the current ClassLoader
         if (is == null) {
             ClassLoader current = ObjectFactory.class.getClassLoader();
             if (cl != current) {
                 cl = current;
-                is = ss.getResourceAsStream(cl, serviceId);
+                is = SecuritySupport.getResourceAsStream(cl, serviceId);
             }
         }
 
