@@ -16,11 +16,11 @@
  */
 package org.apache.wml.dom;
 
-import org.apache.wml.*;
+import org.apache.wml.WMLAccessElement;
 
 /**
  * @xerces.internal
- * @version $Id: WMLAccessElementImpl.java 447257 2006-09-18 05:40:07Z mrglavas $
+ * @version $Id: WMLAccessElementImpl.java 661560 2008-05-30 03:22:25Z mrglavas $
  * @author <a href="mailto:david@topware.com.tw">David Li</a>
  */
 public class WMLAccessElementImpl extends WMLElementImpl implements WMLAccessElement {
