@@ -18,6 +18,7 @@
 package org.apache.xerces.dom;
 
 import java.io.IOException;
+import java.util.ArrayList;
 import java.util.Vector;
 
 import org.apache.xerces.impl.Constants;
@@ -84,7 +85,7 @@ import org.w3c.dom.Text;
  * 
  * @author Elena Litani, IBM
  * @author Neeraj Bajaj, Sun Microsystems, inc.
- * @version $Id: DOMNormalizer.java 541770 2007-05-25 20:17:48Z mrglavas $
+ * @version $Id: DOMNormalizer.java 677489 2008-07-17 02:34:20Z mrglavas $
  */
 public class DOMNormalizer implements XMLDocumentHandler {
 
@@ -136,15 +137,14 @@ public class DOMNormalizer implements XMLDocumentHandler {
     protected final NamespaceContext fLocalNSBinder = new NamespaceSupport();
 
     /** list of attributes */
-    protected final Vector fAttributeList = new Vector(5,10);
-
+    protected final ArrayList fAttributeList = new ArrayList(5);
 
     /** DOM Locator -  for namespace fixup algorithm */
     protected final DOMLocatorImpl fLocator = new DOMLocatorImpl();
 
     /** for setting the PSVI */
     protected Node fCurrentNode = null;
-    private QName fAttrQName = new QName();
+    private final QName fAttrQName = new QName();
     
     // attribute value normalization
     final XMLString fNormalizedValue = new XMLString(new char[16], 0, 0);
@@ -877,7 +877,7 @@ public class DOMNormalizer implements XMLDocumentHandler {
             // clone content of the attributes
             attributes.cloneMap(fAttributeList);
             for (int i = 0; i < fAttributeList.size(); i++) {
-                Attr attr = (Attr) fAttributeList.elementAt(i);
+                Attr attr = (Attr) fAttributeList.get(i);
                 fLocator.fRelatedNode = attr;
 
                 if (DEBUG) {
