@@ -22,13 +22,13 @@ import java.io.ObjectOutputStream;
 import java.io.Serializable;
 import java.util.Hashtable;
 
-import org.w3c.dom.UserDataHandler;
 import org.w3c.dom.DOMException;
 import org.w3c.dom.Document;
 import org.w3c.dom.DocumentType;
 import org.w3c.dom.NamedNodeMap;
 import org.w3c.dom.Node;
 import org.w3c.dom.NodeList;
+import org.w3c.dom.UserDataHandler;
 import org.w3c.dom.events.Event;
 import org.w3c.dom.events.EventListener;
 import org.w3c.dom.events.EventTarget;
@@ -75,7 +75,7 @@ import org.w3c.dom.events.EventTarget;
  *
  * @author Arnaud  Le Hors, IBM
  * @author Joe Kesselman, IBM
- * @version $Id: NodeImpl.java 449328 2006-09-23 22:58:23Z mrglavas $
+ * @version $Id: NodeImpl.java 814769 2009-09-14 18:25:54Z mrglavas $
  * @since  PR-DOM-Level-1-19980818.
  */
 public abstract class NodeImpl
@@ -1540,12 +1540,12 @@ public abstract class NodeImpl
                             if (specifiedPrefix == null &&
                                 attr.getNodeName().equals("xmlns")) {
                                 // default namespace
-                                return value;
+                                return value.length() > 0 ? value : null;
                             } else if (attrPrefix !=null && 
                                        attrPrefix.equals("xmlns") &&
                                        attr.getLocalName().equals(specifiedPrefix)) {
                                 // non default namespace
-                                return value;
+                                return value.length() > 0 ? value : null;
                             }
                         }
                     }
@@ -1586,15 +1586,14 @@ public abstract class NodeImpl
         }
     }
 
-
-    Node getElementAncestor (Node currentNode){
+    Node getElementAncestor(Node currentNode) {
         Node parent = currentNode.getParentNode();
-        if (parent != null) {
+        while (parent != null) {
             short type = parent.getNodeType();
             if (type == Node.ELEMENT_NODE) {
                 return parent;
             }
-            return getElementAncestor(parent);
+            parent = parent.getParentNode();
         }
         return null;
     }
