@@ -59,7 +59,7 @@ import org.apache.xerces.xni.parser.XMLInputSource;
  * @author Glenn Marcy, IBM
  * @author Eric Ye, IBM
  *
- * @version $Id: XMLDTDScannerImpl.java 572055 2007-09-02 17:55:43Z mrglavas $
+ * @version $Id: XMLDTDScannerImpl.java 965250 2010-07-18 16:04:58Z mrglavas $
  */
 public class XMLDTDScannerImpl
     extends XMLScanner
@@ -1400,7 +1400,7 @@ public class XMLDTDScannerImpl
                 isPEDecl = true;
             }
             else if (scanningInternalSubset()) {
-                reportFatalError("MSG_SPACE_REQUIRED_BEFORE_ENTITY_NAME_IN_ENTITYDECL",
+                reportFatalError("MSG_SPACE_REQUIRED_BEFORE_ENTITY_NAME_IN_PEDECL",
                                  null);
                 isPEDecl = true;
             }
@@ -1469,7 +1469,7 @@ public class XMLDTDScannerImpl
             if(fNamespaces && fEntityScanner.peekChar() == ':') {
                 fEntityScanner.scanChar();
                 XMLStringBuffer colonName = new XMLStringBuffer(name);
-                colonName.append(":");
+                colonName.append(':');
                 String str = fEntityScanner.scanName();
                 if (str != null)
                     colonName.append(str);
@@ -1720,7 +1720,7 @@ public class XMLDTDScannerImpl
             if(fNamespaces && fEntityScanner.peekChar() == ':') {
                 fEntityScanner.scanChar();
                 XMLStringBuffer colonName = new XMLStringBuffer(name);
-                colonName.append(":");
+                colonName.append(':');
                 colonName.append(fEntityScanner.scanName());
                 reportFatalError("ColonNotLegalWithNS", new Object[] {colonName.toString()});
                 skipSeparator(true, !scanningInternalSubset());
