@@ -1,58 +1,18 @@
 /*
- * The Apache Software License, Version 1.1
- *
- *
- * Copyright (c) 1999-2003 The Apache Software Foundation.
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Xerces" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation and was
- * originally based on software copyright (c) 1999, International
- * Business Machines, Inc., http://www.apache.org.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 package org.apache.xerces.impl;
@@ -73,16 +33,27 @@ import org.apache.xerces.xni.XMLString;
 /**
  * Implements the entity scanner methods.
  *
+ * @xerces.internal
+ * 
  * @author Andy Clark, IBM
  * @author Neil Graham, IBM
- * @version $Id: XMLEntityScanner.java,v 1.19 2003/11/13 18:45:59 mrglavas Exp $
+ * @version $Id: XMLEntityScanner.java 616011 2008-01-28 19:33:38Z mrglavas $
  */
-
 public class XMLEntityScanner implements XMLLocator {
 
     // constants
     private static final boolean DEBUG_ENCODINGS = false;
     private static final boolean DEBUG_BUFFER = false;
+    
+    /**
+     * To signal the end of the document entity, this exception will be thrown.
+     */
+    private static final EOFException END_OF_DOCUMENT_ENTITY = new EOFException() {
+        private static final long serialVersionUID = 980337771224675268L;
+        public Throwable fillInStackTrace() {
+            return this;
+        }
+    };
 
     //
     // Data
@@ -116,7 +87,7 @@ public class XMLEntityScanner implements XMLLocator {
      * Returns the base system identifier of the currently scanned
      * entity, or null if none is available.
      */
-    public String getBaseSystemId() {
+    public final String getBaseSystemId() {
         return (fCurrentEntity != null && fCurrentEntity.entityLocation != null) ? fCurrentEntity.entityLocation.getExpandedSystemId() : null;
     } // getBaseSystemId():String
 
@@ -138,7 +109,7 @@ public class XMLEntityScanner implements XMLLocator {
      *
      * @see org.apache.xerces.util.EncodingMap
      */
-    public void setEncoding(String encoding) throws IOException {
+    public final void setEncoding(String encoding) throws IOException {
 
         if (DEBUG_ENCODINGS) {
             System.out.println("$$$ setEncoding: "+encoding);
@@ -194,9 +165,20 @@ public class XMLEntityScanner implements XMLLocator {
         }
 
     } // setEncoding(String)
+    
+    /**
+     * Sets the XML version. This method is used by the
+     * scanners to report the value of the version pseudo-attribute
+     * in an XML or text declaration.
+     *
+     * @param xmlVersion the XML version of the current entity
+     */
+    public final void setXMLVersion(String xmlVersion) {
+        fCurrentEntity.xmlVersion = xmlVersion;
+    } // setXMLVersion(String)
 
     /** Returns true if the current entity being scanned is external. */
-    public boolean isExternal() {
+    public final boolean isExternal() {
         return fCurrentEntity.isExternal();
     } // isExternal():boolean
 
@@ -697,6 +679,7 @@ public class XMLEntityScanner implements XMLLocator {
             fCurrentEntity.ch[0] = fCurrentEntity.ch[fCurrentEntity.count - 1];
             load(1, false);
             fCurrentEntity.position = 0;
+            fCurrentEntity.startPosition = 0;
         }
 
         // normalize newlines
@@ -718,7 +701,9 @@ public class XMLEntityScanner implements XMLLocator {
                     fCurrentEntity.columnNumber = 1;
                     if (fCurrentEntity.position == fCurrentEntity.count) {
                         offset = 0;
+                        fCurrentEntity.baseCharOffset += (fCurrentEntity.position - fCurrentEntity.startPosition);
                         fCurrentEntity.position = newlines;
+                        fCurrentEntity.startPosition = newlines;
                         if (load(newlines, false)) {
                             break;
                         }
@@ -738,7 +723,9 @@ public class XMLEntityScanner implements XMLLocator {
                     fCurrentEntity.columnNumber = 1;
                     if (fCurrentEntity.position == fCurrentEntity.count) {
                         offset = 0;
+                        fCurrentEntity.baseCharOffset += (fCurrentEntity.position - fCurrentEntity.startPosition);
                         fCurrentEntity.position = newlines;
+                        fCurrentEntity.startPosition = newlines;
                         if (load(newlines, false)) {
                             break;
                         }
@@ -846,6 +833,7 @@ public class XMLEntityScanner implements XMLLocator {
             fCurrentEntity.ch[0] = fCurrentEntity.ch[fCurrentEntity.count - 1];
             load(1, false);
             fCurrentEntity.position = 0;
+            fCurrentEntity.startPosition = 0;
         }
 
         // normalize newlines
@@ -867,7 +855,9 @@ public class XMLEntityScanner implements XMLLocator {
                     fCurrentEntity.columnNumber = 1;
                     if (fCurrentEntity.position == fCurrentEntity.count) {
                         offset = 0;
+                        fCurrentEntity.baseCharOffset += (fCurrentEntity.position - fCurrentEntity.startPosition);
                         fCurrentEntity.position = newlines;
+                        fCurrentEntity.startPosition = newlines;
                         if (load(newlines, false)) {
                             break;
                         }
@@ -888,7 +878,9 @@ public class XMLEntityScanner implements XMLLocator {
                     fCurrentEntity.columnNumber = 1;
                     if (fCurrentEntity.position == fCurrentEntity.count) {
                         offset = 0;
+                        fCurrentEntity.baseCharOffset += (fCurrentEntity.position - fCurrentEntity.startPosition);
                         fCurrentEntity.position = newlines;
+                        fCurrentEntity.startPosition = newlines;
                         if (load(newlines, false)) {
                             break;
                         }
@@ -978,7 +970,7 @@ public class XMLEntityScanner implements XMLLocator {
      *
      * @param delimiter The string that signifies the end of the character
      *                  data to be scanned.
-     * @param data      The data structure to fill.
+     * @param buffer    The XMLStringBuffer to fill.
      *
      * @return Returns true if there is more data to scan, false otherwise.
      *
@@ -1017,7 +1009,7 @@ public class XMLEntityScanner implements XMLLocator {
 
         boolean bNextEntity = false;
 
-        while ((fCurrentEntity.position >= fCurrentEntity.count - delimLen)
+        while ((fCurrentEntity.position > fCurrentEntity.count - delimLen)
             && (!bNextEntity))
         {
           System.arraycopy(fCurrentEntity.ch,
@@ -1028,14 +1020,17 @@ public class XMLEntityScanner implements XMLLocator {
 
           bNextEntity = load(fCurrentEntity.count - fCurrentEntity.position, false);
           fCurrentEntity.position = 0;
+          fCurrentEntity.startPosition = 0;
         }
 
-        if (fCurrentEntity.position >= fCurrentEntity.count - delimLen) {
-            // something must be wrong with the input:  e.g., file ends  an unterminated comment
+        if (fCurrentEntity.position > fCurrentEntity.count - delimLen) {
+            // something must be wrong with the input:  e.g., file ends in an unterminated comment
             int length = fCurrentEntity.count - fCurrentEntity.position;
             buffer.append (fCurrentEntity.ch, fCurrentEntity.position, length); 
             fCurrentEntity.columnNumber += fCurrentEntity.count;
+            fCurrentEntity.baseCharOffset += (fCurrentEntity.position - fCurrentEntity.startPosition);
             fCurrentEntity.position = fCurrentEntity.count;
+            fCurrentEntity.startPosition = fCurrentEntity.count;
             load(0,true);
             return false;
         }
@@ -1058,7 +1053,9 @@ public class XMLEntityScanner implements XMLLocator {
                     fCurrentEntity.columnNumber = 1;
                     if (fCurrentEntity.position == fCurrentEntity.count) {
                         offset = 0;
+                        fCurrentEntity.baseCharOffset += (fCurrentEntity.position - fCurrentEntity.startPosition);
                         fCurrentEntity.position = newlines;
+                        fCurrentEntity.startPosition = newlines;
                         if (load(newlines, false)) {
                             break;
                         }
@@ -1078,7 +1075,9 @@ public class XMLEntityScanner implements XMLLocator {
                     fCurrentEntity.columnNumber = 1;
                     if (fCurrentEntity.position == fCurrentEntity.count) {
                         offset = 0;
+                        fCurrentEntity.baseCharOffset += (fCurrentEntity.position - fCurrentEntity.startPosition);
                         fCurrentEntity.position = newlines;
+                        fCurrentEntity.startPosition = newlines;
                         fCurrentEntity.count = newlines;
                         if (load(newlines, false)) {
                             break;
@@ -1272,10 +1271,12 @@ public class XMLEntityScanner implements XMLLocator {
                     if (fCurrentEntity.position == fCurrentEntity.count - 1) {
                         fCurrentEntity.ch[0] = (char)c;
                         entityChanged = load(1, true);
-                        if (!entityChanged)
+                        if (!entityChanged) {
                             // the load change the position to be 1,
                             // need to restore it when entity not changed
                             fCurrentEntity.position = 0;
+                            fCurrentEntity.startPosition = 0;
+                        }
                     }
                     if (c == '\r' && external) {
                         // REVISIT: Does this need to be updated to fix the
@@ -1338,7 +1339,7 @@ public class XMLEntityScanner implements XMLLocator {
      *
      * @see org.apache.xerces.util.XMLChar#isSpace
      */
-    public boolean skipDeclSpaces() throws IOException {
+    public final boolean skipDeclSpaces() throws IOException {
         if (DEBUG_BUFFER) {
             System.out.print("(skipDeclSpaces: ");
             XMLEntityManager.print(fCurrentEntity);
@@ -1363,10 +1364,12 @@ public class XMLEntityScanner implements XMLLocator {
                     if (fCurrentEntity.position == fCurrentEntity.count - 1) {
                         fCurrentEntity.ch[0] = (char)c;
                         entityChanged = load(1, true);
-                        if (!entityChanged)
+                        if (!entityChanged) {
                             // the load change the position to be 1,
                             // need to restore it when entity not changed
                             fCurrentEntity.position = 0;
+                            fCurrentEntity.startPosition = 0;
+                        }
                     }
                     if (c == '\r' && external) {
                         // REVISIT: Does this need to be updated to fix the
@@ -1455,6 +1458,7 @@ public class XMLEntityScanner implements XMLLocator {
                 // REVISIT: Can a string to be skipped cross an
                 //          entity boundary? -Ac
                 if (load(i + 1, false)) {
+                    fCurrentEntity.startPosition -= i + 1; 
                     fCurrentEntity.position -= i + 1;
                     if (DEBUG_BUFFER) {
                         System.out.print(")skipString, \""+s+"\": ");
@@ -1489,7 +1493,7 @@ public class XMLEntityScanner implements XMLLocator {
      * @return A string containing the public identifier, or
      *         null if none is available.
      */
-    public String getPublicId() {
+    public final String getPublicId() {
         return (fCurrentEntity != null && fCurrentEntity.entityLocation != null) ? fCurrentEntity.entityLocation.getPublicId() : null;
     } // getPublicId():String
 
@@ -1506,7 +1510,7 @@ public class XMLEntityScanner implements XMLLocator {
      * @return A string containing the expanded system identifier, or null
      *         if none is available.
      */
-    public String getExpandedSystemId() {
+    public final String getExpandedSystemId() {
         if (fCurrentEntity != null) {
             if (fCurrentEntity.entityLocation != null &&
                     fCurrentEntity.entityLocation.getExpandedSystemId() != null ) {
@@ -1530,7 +1534,7 @@ public class XMLEntityScanner implements XMLLocator {
      * @return A string containing the literal system identifier, or null
      *         if none is available.
      */
-    public String getLiteralSystemId() {
+    public final String getLiteralSystemId() {
         if (fCurrentEntity != null) {
             if (fCurrentEntity.entityLocation != null &&
                     fCurrentEntity.entityLocation.getLiteralSystemId() != null ) {
@@ -1545,7 +1549,7 @@ public class XMLEntityScanner implements XMLLocator {
     } // getLiteralSystemId():String
 
     /**
-     * Return the line number where the current document event ends.
+     * Returns the line number where the current document event ends.
      * <p>
      * <strong>Warning:</strong> The return value from the method
      * is intended only as an approximation for the sake of error
@@ -1556,13 +1560,13 @@ public class XMLEntityScanner implements XMLLocator {
      * in the document entity or external parsed entity where the
      * markup triggering the event appears.
      * <p>
-     * If possible, the SAX driver should provide the line position
-     * of the first character after the text associated with the document
-     * event.  The first line in the document is line 1.
+     * If possible, the line position of the first character after the 
+     * text associated with the document event should be provided.
+     * The first line in the document is line 1.
      *
      * @return The line number, or -1 if none is available.
      */
-    public int getLineNumber() {
+    public final int getLineNumber() {
         if (fCurrentEntity != null) {
             if (fCurrentEntity.isExternal()) {
                 return fCurrentEntity.lineNumber;
@@ -1578,7 +1582,7 @@ public class XMLEntityScanner implements XMLLocator {
     } // getLineNumber():int
 
     /**
-     * Return the column number where the current document event ends.
+     * Returns the column number where the current document event ends.
      * <p>
      * <strong>Warning:</strong> The return value from the method
      * is intended only as an approximation for the sake of error
@@ -1589,17 +1593,13 @@ public class XMLEntityScanner implements XMLLocator {
      * in the document entity or external parsed entity where the
      * markup triggering the event appears.
      * <p>
-     * If possible, the SAX driver should provide the line position
-     * of the first character after the text associated with the document
-     * event.
-     * <p>
-     * If possible, the SAX driver should provide the line position
-     * of the first character after the text associated with the document
-     * event.  The first column in each line is column 1.
+     * If possible, the line position of the first character after the 
+     * text associated with the document event should be provided.
+     * The first column in each line is column 1.
      *
      * @return The column number, or -1 if none is available.
      */
-    public int getColumnNumber() {
+    public final int getColumnNumber() {
         if (fCurrentEntity != null) {
             if (fCurrentEntity.isExternal()) {
                 return fCurrentEntity.columnNumber;
@@ -1613,77 +1613,88 @@ public class XMLEntityScanner implements XMLLocator {
         return -1;
     } // getColumnNumber():int
     
-    /** Returns the encoding of the current entity.  
+    /**
+     * Returns the character offset where the current document event ends.
+     * <p>
+     * <strong>Warning:</strong> The return value from the method
+     * is intended only as an approximation for the sake of error
+     * reporting; it is not intended to provide sufficient information
+     * to edit the character content of the original XML document.
+     * <p>
+     * The return value is an approximation of the character offset
+     * in the document entity or external parsed entity where the
+     * markup triggering the event appears.
+     * <p>
+     * If possible, the character offset of the first character after the 
+     * text associated with the document event should be provided.
+     *
+     * @return The character offset, or -1 if none is available.
+     */
+    public final int getCharacterOffset() {
+        if (fCurrentEntity != null) {
+            if (fCurrentEntity.isExternal()) {
+                return fCurrentEntity.baseCharOffset + (fCurrentEntity.position - fCurrentEntity.startPosition);
+            }
+            else {
+                // ask current entity to find appropriate character offset
+                return fCurrentEntity.getCharacterOffset();
+            }
+        }
+        
+        return -1;
+    } // getCharacterOffset():int
+    
+    /** 
+     * Returns the encoding of the current entity.  
      * Note that, for a given entity, this value can only be
      * considered final once the encoding declaration has been read (or once it
      * has been determined that there is no such declaration) since, no encoding
      * having been specified on the XMLInputSource, the parser
      * will make an initial "guess" which could be in error. 
      */
-    public String getEncoding() {
+    public final String getEncoding() {
         if (fCurrentEntity != null) {
             if (fCurrentEntity.isExternal()) {
                 return fCurrentEntity.encoding;
             }
             else {
-                // ask current entity to find appropriate column number
+                // ask current entity to find appropriate encoding
                 return fCurrentEntity.getEncoding();
             }
         }
-
         return null;
     } // getEncoding():String
     
     /**
-     * @see org.apache.xerces.xni.XMLLocator#setColumnNumber(int)
+     * Returns the XML version of the current entity. This will normally be the
+     * value from the XML or text declaration or defaulted by the parser. Note that
+     * that this value may be different than the version of the processing rules 
+     * applied to the current entity. For instance, an XML 1.1 document may refer to
+     * XML 1.0 entities. In such a case the rules of XML 1.1 are applied to the entire 
+     * document. Also note that, for a given entity, this value can only be considered
+     * final once the XML or text declaration has been read or once it has been
+     * determined that there is no such declaration.
      */
-    public void setColumnNumber(int col) {
-        // no-op
-    }
-
-    /**
-     * @see org.apache.xerces.xni.XMLLocator#setLineNumber(int)
-     */
-    public void setLineNumber(int line) {
-        //no-op
-    }
+    public final String getXMLVersion() {
+        if (fCurrentEntity != null) {
+            if (fCurrentEntity.isExternal()) {
+                return fCurrentEntity.xmlVersion;
+            }
+            else {
+                // ask current entity to find the appropriate XML version
+                return fCurrentEntity.getXMLVersion();
+            }
+        }
+        return null;
+    } // getXMLVersion():String
     
-        /**
-     * @see org.apache.xerces.xni.XMLResourceIdentifier#setBaseSystemId(String)
-     */
-    public void setBaseSystemId(String systemId) {        
-        //no-op
-    }
-
-
-    /**
-     * @see org.apache.xerces.xni.XMLResourceIdentifier#setExpandedSystemId(String)
-     */
-    public void setExpandedSystemId(String systemId) {
-        //no-op
-    }
-
-    /**
-     * @see org.apache.xerces.xni.XMLResourceIdentifier#setLiteralSystemId(String)
-     */
-    public void setLiteralSystemId(String systemId) {
-        //no-op
-    }
-
-    /**
-     * @see org.apache.xerces.xni.XMLResourceIdentifier#setPublicId(String)
-     */
-    public void setPublicId(String publicId) {
-        //no-op
-    }
-
     // allow entity manager to tell us what the current entityis:
-    public void setCurrentEntity(XMLEntityManager.ScannedEntity ent) {
+    public final void setCurrentEntity(XMLEntityManager.ScannedEntity ent) {
         fCurrentEntity = ent;
     }
 
     // set buffer size:
-    public void setBufferSize(int size) {
+    public final void setBufferSize(int size) {
         // REVISIT: Buffer size passed to entity scanner 
         // was not being kept in synch with the actual size
         // of the buffers in each scanned entity. If any
@@ -1698,7 +1709,7 @@ public class XMLEntityScanner implements XMLLocator {
     }
 
     // reset what little state we have...
-    public void reset(SymbolTable symbolTable, XMLEntityManager entityManager,
+    public final void reset(SymbolTable symbolTable, XMLEntityManager entityManager,
                         XMLErrorReporter reporter) {
         fCurrentEntity = null;
         fSymbolTable = symbolTable;
@@ -1732,10 +1743,12 @@ public class XMLEntityScanner implements XMLLocator {
             System.out.println();
         }
 
+        fCurrentEntity.baseCharOffset += (fCurrentEntity.position - fCurrentEntity.startPosition);
         // read characters
-        int length = fCurrentEntity.mayReadChunks?
-                (fCurrentEntity.ch.length - offset):
-                (XMLEntityManager.DEFAULT_XMLDECL_BUFFER_SIZE);
+        int length = fCurrentEntity.ch.length - offset;
+        if (!fCurrentEntity.mayReadChunks && length > XMLEntityManager.DEFAULT_XMLDECL_BUFFER_SIZE) {
+            length = XMLEntityManager.DEFAULT_XMLDECL_BUFFER_SIZE;
+        }
         if (DEBUG_BUFFER) System.out.println("  length to try to read: "+length);
         int count = fCurrentEntity.reader.read(fCurrentEntity.ch, offset, length);
         if (DEBUG_BUFFER) System.out.println("  length actually read:  "+count);
@@ -1746,6 +1759,7 @@ public class XMLEntityScanner implements XMLLocator {
             if (count != 0) {
                 fCurrentEntity.count = count + offset;
                 fCurrentEntity.position = offset;
+                fCurrentEntity.startPosition = offset;
             }
         }
 
@@ -1753,11 +1767,12 @@ public class XMLEntityScanner implements XMLLocator {
         else {
             fCurrentEntity.count = offset;
             fCurrentEntity.position = offset;
+            fCurrentEntity.startPosition = offset;
             entityChanged = true;
             if (changeEntity) {
                 fEntityManager.endEntity();
                 if (fCurrentEntity == null) {
-                    throw new EOFException();
+                    throw END_OF_DOCUMENT_ENTITY;
                 }
                 // handle the trailing edges
                 if (fCurrentEntity.position == fCurrentEntity.count) {
@@ -1775,6 +1790,5 @@ public class XMLEntityScanner implements XMLLocator {
 
     } // load(int, boolean):boolean
 
-
 } // class XMLEntityScanner
 
