@@ -1,9 +1,10 @@
 /*
- * Copyright 1999-2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -55,7 +56,7 @@ import org.apache.xerces.xni.parser.XMLConfigurationException;
  * @author Arnaud  Le Hors, IBM
  * @author Eric Ye, IBM
  *
- * @version $Id: XMLScanner.java 320089 2004-10-04 21:45:49Z mrglavas $
+ * @version $Id: XMLScanner.java 965250 2010-07-18 16:04:58Z mrglavas $
  */
 public abstract class XMLScanner 
     implements XMLComponent {
@@ -185,19 +186,19 @@ public abstract class XMLScanner
     //       possibility.
 
     /** String. */
-    private XMLString fString = new XMLString();
+    private final XMLString fString = new XMLString();
 
     /** String buffer. */
-    private XMLStringBuffer fStringBuffer = new XMLStringBuffer();
+    private final XMLStringBuffer fStringBuffer = new XMLStringBuffer();
 
     /** String buffer. */
-    private XMLStringBuffer fStringBuffer2 = new XMLStringBuffer();
+    private final XMLStringBuffer fStringBuffer2 = new XMLStringBuffer();
 
     /** String buffer. */
-    private XMLStringBuffer fStringBuffer3 = new XMLStringBuffer();
+    private final XMLStringBuffer fStringBuffer3 = new XMLStringBuffer();
 
     // temporary location for Resource identification information.
-    protected XMLResourceIdentifierImpl fResourceIdentifier = new XMLResourceIdentifierImpl();
+    protected final XMLResourceIdentifierImpl fResourceIdentifier = new XMLResourceIdentifierImpl();
 
     //
     // XMLComponent methods
@@ -662,7 +663,7 @@ public abstract class XMLScanner
                 if(fNamespaces && fEntityScanner.peekChar() == ':') { 
                     fEntityScanner.scanChar();
                     XMLStringBuffer colonName = new XMLStringBuffer(target);
-                    colonName.append(":");
+                    colonName.append(':');
                     String str = fEntityScanner.scanName();
                     if (str != null)
                         colonName.append(str);
@@ -1026,6 +1027,14 @@ public abstract class XMLScanner
                     if (XMLChar.isMarkup(c) || c == ']') {
                         fStringBuffer.append((char)fEntityScanner.scanChar());
                     }
+                    else if (XMLChar.isHighSurrogate(c)) {
+                        scanSurrogates(fStringBuffer);
+                    }
+                    else if (isInvalidLiteral(c)) {
+                        reportFatalError("InvalidCharInSystemID",
+                                new Object[] { Integer.toHexString(c) }); 
+                        fEntityScanner.scanChar();
+                    }
                 } while (fEntityScanner.scanLiteral(quote, ident) != quote);
                 fStringBuffer.append(ident);
                 ident = fStringBuffer;
