@@ -17,9 +17,8 @@
 
 package org.apache.xerces.impl.dv;
 
-import java.util.ResourceBundle;
-import java.util.PropertyResourceBundle;
 import java.util.MissingResourceException;
+import java.util.ResourceBundle;
 
 /**
  * Base class for datatype exceptions. For DTD types, the exception can be
@@ -31,7 +30,7 @@ import java.util.MissingResourceException;
  *
  * @author Sandy Gao, IBM
  *
- * @version $Id: DatatypeException.java 572086 2007-09-02 18:26:08Z mrglavas $
+ * @version $Id: DatatypeException.java 809242 2009-08-30 03:34:31Z mrglavas $
  */
 public class DatatypeException extends Exception {
 
@@ -82,7 +81,7 @@ public class DatatypeException extends Exception {
      */
     public String getMessage() {
         ResourceBundle resourceBundle = null;
-        resourceBundle = PropertyResourceBundle.getBundle("org.apache.xerces.impl.msg.XMLSchemaMessages");
+        resourceBundle = ResourceBundle.getBundle("org.apache.xerces.impl.msg.XMLSchemaMessages");
         if (resourceBundle == null)
             throw new MissingResourceException("Property file not found!", "org.apache.xerces.impl.msg.XMLSchemaMessages", key);
 
