@@ -26,7 +26,7 @@ import java.util.Locale;
  * @xerces.internal
  *
  * @author TAMURA Kent &lt;kent@trl.ibm.co.jp&gt;
- * @version $Id: ParserForXMLSchema.java 571468 2007-08-31 14:01:58Z knoaman $
+ * @version $Id: ParserForXMLSchema.java 831926 2009-11-02 15:38:53Z knoaman $
  */
 class ParserForXMLSchema extends RegexParser {
 
@@ -34,7 +34,7 @@ class ParserForXMLSchema extends RegexParser {
         //this.setLocale(Locale.getDefault());
     }
     public ParserForXMLSchema(Locale locale) {
-        //this.setLocale(locale);
+        super(locale);
     }
 
     Token processCaret() throws ParseException {
@@ -250,14 +250,24 @@ class ParserForXMLSchema extends RegexParser {
                     if (c == '-' && this.chardata != ']' && !firstloop)  throw this.ex("parser.cc.8", this.offset-2);	// if regex = '[-]' then invalid
                 }
                 if (this.read() != T_CHAR || this.chardata != '-' || c == '-' && firstloop) { // Here is no '-'.
-                    tok.addRange(c, c);
+                    if (!this.isSet(RegularExpression.IGNORE_CASE) || c > 0xffff) {
+                        tok.addRange(c, c);
+                    }
+                    else {
+                        addCaseInsensitiveChar(tok, c);
+                    }
                 } else {                        // Found '-'
                                                 // Is this '-' is a from-to token??
                     this.next(); // Skips '-'
                     if ((type = this.read()) == T_EOF)  throw this.ex("parser.cc.2", this.offset);
                                                 // c '-' ']' -> '-' is a single-range.
                     if(type == T_CHAR && this.chardata == ']') {				// if - is at the last position of the group
-                    	tok.addRange(c, c);
+                        if (!this.isSet(RegularExpression.IGNORE_CASE) || c > 0xffff) {
+                    	    tok.addRange(c, c);
+                        }
+                        else {
+                            addCaseInsensitiveChar(tok, c);
+                        }
                     	tok.addRange('-', '-');
                     }
                     else if (type == T_XMLSCHEMA_CC_SUBTRACTION) {
@@ -275,7 +285,13 @@ class ParserForXMLSchema extends RegexParser {
                         this.next();
 
                         if (c > rangeend)  throw this.ex("parser.ope.3", this.offset-1);
-                        tok.addRange(c, rangeend);
+                        if (!this.isSet(RegularExpression.IGNORE_CASE) ||
+                                (c > 0xffff && rangeend > 0xffff)) {
+                            tok.addRange(c, rangeend);
+                        }
+                        else {
+                            addCaseInsensitiveCharRange(tok, c, rangeend);
+                        }
                     }
                 }
             }
