@@ -1,9 +1,10 @@
 /*
- * Copyright 2001-2006 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -16,6 +17,8 @@
 
 package org.apache.xerces.impl.xs.traversers;
 
+import java.util.Locale;
+
 import org.apache.xerces.impl.dv.ValidatedInfo;
 import org.apache.xerces.impl.dv.XSSimpleType;
 import org.apache.xerces.impl.xs.SchemaGrammar;
@@ -25,17 +28,18 @@ import org.apache.xerces.impl.xs.XSComplexTypeDecl;
 import org.apache.xerces.impl.xs.XSConstraints;
 import org.apache.xerces.impl.xs.XSElementDecl;
 import org.apache.xerces.impl.xs.XSParticleDecl;
-import org.apache.xerces.xs.XSConstants;
-import org.apache.xerces.xs.XSObject;
-import org.apache.xerces.xs.XSObjectList;
-import org.apache.xerces.xs.XSTypeDefinition;
 import org.apache.xerces.impl.xs.util.XInt;
 import org.apache.xerces.impl.xs.util.XSObjectListImpl;
 import org.apache.xerces.util.DOMUtil;
 import org.apache.xerces.util.SymbolTable;
+import org.apache.xerces.util.XMLChar;
 import org.apache.xerces.xni.QName;
-import org.w3c.dom.Element;
+import org.apache.xerces.xs.XSConstants;
+import org.apache.xerces.xs.XSObject;
+import org.apache.xerces.xs.XSObjectList;
+import org.apache.xerces.xs.XSTypeDefinition;
 import org.w3c.dom.Attr;
+import org.w3c.dom.Element;
 
 /**
  * The element declaration schema component traverser.
@@ -62,7 +66,7 @@ import org.w3c.dom.Attr;
  *
  * @author Sandy Gao, IBM
  *
- * @version $Id: XSDElementTraverser.java 442099 2006-09-11 05:01:27Z mrglavas $
+ * @version $Id: XSDElementTraverser.java 821931 2009-10-05 17:42:57Z sandygao $
  */
 class XSDElementTraverser extends XSDAbstractTraverser {
     
@@ -102,7 +106,7 @@ class XSDElementTraverser extends XSDAbstractTraverser {
         } else {
             particle = new XSParticleDecl();
         }
-        if(fDeferTraversingLocalElements) {
+        if (fDeferTraversingLocalElements) {
             // The only thing we care about now is whether this element has
             // minOccurs=0. This affects (if the element appears in a complex
             // type) whether a type has emptiable content.
@@ -111,7 +115,7 @@ class XSDElementTraverser extends XSDAbstractTraverser {
             if (attr != null) {
                 String min = attr.getValue();
                 try {
-                    int m = Integer.parseInt(min.trim());
+                    int m = Integer.parseInt(XMLChar.trim(min));
                     if (m >= 0)
                         particle.fMinOccurs = m;
                 }
@@ -200,7 +204,7 @@ class XSDElementTraverser extends XSDAbstractTraverser {
             XSObjectList annotations;
             if (annotation != null) {
                 annotations = new XSObjectListImpl();
-                ((XSObjectListImpl) annotations).add(annotation);
+                ((XSObjectListImpl) annotations).addXSObject(annotation);
             } else {
                 annotations = XSObjectListImpl.EMPTY_LIST;
             }
@@ -346,7 +350,7 @@ class XSDElementTraverser extends XSDAbstractTraverser {
         XSObjectList annotations;
         if (annotation != null) {
             annotations = new XSObjectListImpl();
-            ((XSObjectListImpl)annotations).add (annotation);
+            ((XSObjectListImpl)annotations).addXSObject (annotation);
         } else {
             annotations = XSObjectListImpl.EMPTY_LIST;
         }
@@ -375,6 +379,9 @@ class XSDElementTraverser extends XSDAbstractTraverser {
         // Handler type attribute
         if (elementType == null && typeAtt != null) {
             elementType = (XSTypeDefinition)fSchemaHandler.getGlobalDecl(schemaDoc, XSDHandler.TYPEDECL_TYPE, typeAtt, elmDecl);
+            if (elementType == null) {
+                element.fUnresolvedTypeName = typeAtt;
+            }
         }
         
         // Get it from the substitutionGroup declaration
@@ -402,13 +409,13 @@ class XSDElementTraverser extends XSDAbstractTraverser {
                         childName.equals(SchemaSymbols.ELT_UNIQUE)) {
                     // need to set <key>/<unique> to hidden before traversing it,
                     // because it has global scope
-                    DOMUtil.setHidden(child);
+                    DOMUtil.setHidden(child, fSchemaHandler.fHiddenNodes);
                     fSchemaHandler.fUniqueOrKeyTraverser.traverse(child, element, schemaDoc, grammar);
                     if(DOMUtil.getAttrValue(child, SchemaSymbols.ATT_NAME).length() != 0 ) {
                         fSchemaHandler.checkForDuplicateNames(
                                 (schemaDoc.fTargetNamespace == null) ? ","+DOMUtil.getAttrValue(child, SchemaSymbols.ATT_NAME)
                                         : schemaDoc.fTargetNamespace+","+ DOMUtil.getAttrValue(child, SchemaSymbols.ATT_NAME),
-                                        fSchemaHandler.getIDRegistry(), fSchemaHandler.getIDRegistry_sub(),
+                                        fSchemaHandler.ATTRIBUTE_TYPE, fSchemaHandler.getIDRegistry(), fSchemaHandler.getIDRegistry_sub(),
                                         child, schemaDoc);
                     }
                 } else if (childName.equals(SchemaSymbols.ELT_KEYREF)) {
@@ -421,10 +428,6 @@ class XSDElementTraverser extends XSDAbstractTraverser {
             }
         }
         
-        // Step 2: register the element decl to the grammar
-        if (isGlobal && nameAtt != null)
-            grammar.addGlobalElementDecl(element);
-        
         // Step 3: check against schema for schemas
         
         // required attributes
@@ -473,6 +476,7 @@ class XSDElementTraverser extends XSDAbstractTraverser {
             fValidationState.setNamespaceSupport(schemaDoc.fNamespaceSupport);
             if (XSConstraints.ElementDefaultValidImmediate(element.fType, element.fDefault.normalizedValue, fValidationState, element.fDefault) == null) {
                 reportSchemaError ("e-props-correct.2", new Object[]{nameAtt, element.fDefault.normalizedValue}, elmDecl);
+                element.fDefault = null;
                 element.setConstraintType(XSConstants.VC_NONE);
             }
         }
@@ -481,6 +485,7 @@ class XSDElementTraverser extends XSDAbstractTraverser {
         if (element.fSubGroup != null) {
             if (!XSConstraints.checkTypeDerivationOk(element.fType, element.fSubGroup.fType, element.fSubGroup.fFinal)) {
                 reportSchemaError ("e-props-correct.4", new Object[]{nameAtt, subGroupAtt.prefix+":"+subGroupAtt.localpart}, elmDecl);
+                element.fSubGroup = null;
             }
         }
         
@@ -491,6 +496,8 @@ class XSDElementTraverser extends XSDAbstractTraverser {
                     (elementType.getTypeCategory() == XSTypeDefinition.COMPLEX_TYPE &&
                             ((XSComplexTypeDecl)elementType).containsTypeID())) {
                 reportSchemaError ("e-props-correct.5", new Object[]{element.fName}, elmDecl);
+                element.fDefault = null;
+                element.setConstraintType(XSConstants.VC_NONE);
             }
         }
         
@@ -498,11 +505,36 @@ class XSDElementTraverser extends XSDAbstractTraverser {
         if (element.fName == null)
             return null;
         
+        // Step 5: register the element decl to the grammar
+        if (isGlobal) {
+            grammar.addGlobalElementDeclAll(element);
+            
+            if (grammar.getGlobalElementDecl(element.fName) == null) {
+                grammar.addGlobalElementDecl(element);
+            }
+            
+            // we also add the element to the tolerate duplicates list as well
+            final String loc = fSchemaHandler.schemaDocument2SystemId(schemaDoc);
+            final XSElementDecl element2 = grammar.getGlobalElementDecl(element.fName, loc);
+            if (element2 == null) {
+                grammar.addGlobalElementDecl(element, loc);
+            }
+
+            // if we are tolerating duplicates, and we found a duplicate declaration
+            // use the duplicate one instead
+            if (fSchemaHandler.fTolerateDuplicates) {
+                if (element2 != null) {
+                    element = element2;
+                }
+                fSchemaHandler.addGlobalElementDecl(element);
+            }
+        }
+        
         return element;
     }
     
-    void reset(SymbolTable symbolTable, boolean validateAnnotations) {
-        super.reset(symbolTable, validateAnnotations);
+    void reset(SymbolTable symbolTable, boolean validateAnnotations, Locale locale) {
+        super.reset(symbolTable, validateAnnotations, locale);
         fDeferTraversingLocalElements = true;
     } // reset()
     
