@@ -19,6 +19,7 @@ package org.apache.xerces.impl.xs.util;
 
 import java.util.ArrayList;
 
+import org.apache.xerces.impl.Constants;
 import org.apache.xerces.impl.xs.SchemaGrammar;
 import org.apache.xerces.impl.xs.XSModelImpl;
 import org.apache.xerces.util.XMLGrammarPoolImpl;
@@ -31,9 +32,10 @@ import org.apache.xerces.xs.XSModel;
  * 
  * @xerces.internal  
  * 
- * @version $Id: XSGrammarPool.java 446723 2006-09-15 20:37:45Z mrglavas $
+ * @version $Id: XSGrammarPool.java 770963 2009-05-02 15:27:54Z mrglavas $
  */
 public class XSGrammarPool extends XMLGrammarPoolImpl {
+
     /**
      * Return an <code>XSModel</code> that represents components in
      * the schema grammars in this pool implementation.
@@ -41,21 +43,28 @@ public class XSGrammarPool extends XMLGrammarPoolImpl {
      * @return  an <code>XSModel</code> representing this schema grammar
      */
     public XSModel toXSModel() {
+    	return toXSModel(Constants.SCHEMA_VERSION_1_0);
+    }
+    
+    public XSModel toXSModel(short schemaVersion) {
         ArrayList list = new ArrayList();
         for (int i = 0; i < fGrammars.length; i++) {
             for (Entry entry = fGrammars[i] ; entry != null ; entry = entry.next) {
-                if (entry.desc.getGrammarType().equals(XMLGrammarDescription.XML_SCHEMA))
+                if (entry.desc.getGrammarType().equals(XMLGrammarDescription.XML_SCHEMA)) {
                     list.add(entry.grammar);
+                }
             }
         }
-
         int size = list.size();
         if (size == 0) {
-            return null;
+            return toXSModel(new SchemaGrammar[0], schemaVersion);
         }
         SchemaGrammar[] gs = (SchemaGrammar[])list.toArray(new SchemaGrammar[size]);
-        return new XSModelImpl(gs);
+        return toXSModel(gs, schemaVersion);
+    }
+    
+    protected XSModel toXSModel(SchemaGrammar[] grammars, short schemaVersion) {
+        return new XSModelImpl(grammars, schemaVersion);
     }
 
-
 } // class XSGrammarPool
