@@ -1,9 +1,10 @@
 /*
- * Copyright 2001-2006 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -16,13 +17,13 @@
 
 package org.apache.xerces.parsers;
 
+import java.io.CharConversionException;
 import java.io.IOException;
 import java.util.Locale;
 
 import org.apache.xerces.impl.Constants;
-import org.apache.xerces.xs.PSVIProvider;
-import org.apache.xerces.util.EntityResolverWrapper;
 import org.apache.xerces.util.EntityResolver2Wrapper;
+import org.apache.xerces.util.EntityResolverWrapper;
 import org.apache.xerces.util.ErrorHandlerWrapper;
 import org.apache.xerces.util.SAXMessageFormatter;
 import org.apache.xerces.util.SymbolHash;
@@ -43,15 +44,14 @@ import org.apache.xerces.xni.parser.XMLParseException;
 import org.apache.xerces.xni.parser.XMLParserConfiguration;
 import org.apache.xerces.xs.AttributePSVI;
 import org.apache.xerces.xs.ElementPSVI;
+import org.apache.xerces.xs.PSVIProvider;
 import org.xml.sax.AttributeList;
-import org.xml.sax.Attributes;
 import org.xml.sax.ContentHandler;
 import org.xml.sax.DTDHandler;
 import org.xml.sax.DocumentHandler;
 import org.xml.sax.EntityResolver;
 import org.xml.sax.ErrorHandler;
 import org.xml.sax.InputSource;
-import org.xml.sax.Locator;
 import org.xml.sax.Parser;
 import org.xml.sax.SAXException;
 import org.xml.sax.SAXNotRecognizedException;
@@ -63,7 +63,7 @@ import org.xml.sax.ext.DeclHandler;
 import org.xml.sax.ext.EntityResolver2;
 import org.xml.sax.ext.LexicalHandler;
 import org.xml.sax.ext.Locator2;
-import org.xml.sax.helpers.LocatorImpl;
+import org.xml.sax.ext.Locator2Impl;
 
 /**
  * This is the base class of all SAX parsers. It implements both the
@@ -73,7 +73,7 @@ import org.xml.sax.helpers.LocatorImpl;
  * @author Arnaud Le Hors, IBM
  * @author Andy Clark, IBM
  *
- * @version $Id: AbstractSAXParser.java 390944 2006-04-03 02:46:45Z mrglavas $
+ * @version $Id: AbstractSAXParser.java 965250 2010-07-18 16:04:58Z mrglavas $
  */
 public abstract class AbstractSAXParser
     extends AbstractXMLDocumentParser
@@ -178,7 +178,7 @@ public abstract class AbstractSAXParser
     /** Lexical handler. */
     protected LexicalHandler fLexicalHandler;
 
-    protected QName fQName = new QName();
+    protected final QName fQName = new QName();
 
     // state
 
@@ -196,12 +196,6 @@ public abstract class AbstractSAXParser
     private final AttributesProxy fAttributesProxy = new AttributesProxy();
     private Augmentations fAugmentations = null;
 
-
-    // temporary buffer for sending normalized values
-    // REVISIT: what should be the size of the buffer?
-    private static final int BUFFER_SIZE = 20;
-    private char[] fCharBuffer =  new char[BUFFER_SIZE];
-
     // allows us to keep track of whether an attribute has
     // been declared twice, so that we can avoid exposing the
     // second declaration to any registered DeclHandler
@@ -268,7 +262,11 @@ public abstract class AbstractSAXParser
                 if (locator != null) {
                     fDocumentHandler.setDocumentLocator(new LocatorProxy(locator));
                 }
-                fDocumentHandler.startDocument();
+                // The application may have set the DocumentHandler to null
+                // within setDocumentLocator() so we need to check again.
+                if (fDocumentHandler != null) {
+                    fDocumentHandler.startDocument();
+                }
             }
 
             // SAX2
@@ -276,7 +274,11 @@ public abstract class AbstractSAXParser
                 if (locator != null) {
                     fContentHandler.setDocumentLocator(new LocatorProxy(locator));
                 }
-                fContentHandler.startDocument();
+                // The application may have set the ContentHandler to null
+                // within setDocumentLocator() so we need to check again.
+                if (fContentHandler != null) {
+                    fContentHandler.startDocument();
+                }  
             }
         }
         catch (SAXException e) {
@@ -916,7 +918,7 @@ public abstract class AbstractSAXParser
             // SAX2 extension
             if (fDeclHandler != null) {
                 // used as a key to detect duplicate attribute definitions.
-                String elemAttr = new StringBuffer(elementName).append("<").append(attributeName).toString();
+                String elemAttr = new StringBuffer(elementName).append('<').append(attributeName).toString();
                 if(fDeclaredAttrs.get(elemAttr) != null) {
                     // we aren't permitted to return duplicate attribute definitions
                     return;
@@ -931,7 +933,7 @@ public abstract class AbstractSAXParser
                       str.append(" (");
                     }
                     else {
-                      str.append("(");
+                      str.append('(');
                     }
                     for (int i = 0; i < enumeration.length; i++) {
                         str.append(enumeration[i]);
@@ -1130,27 +1132,23 @@ public abstract class AbstractSAXParser
         // wrap XNI exceptions as SAX exceptions
         catch (XMLParseException e) {
             Exception ex = e.getException();
-            if (ex == null) {
-                // must be a parser exception; mine it for locator info and throw
-                // a SAXParseException
-                LocatorImpl locatorImpl = new LocatorImpl(){
-                    public String getXMLVersion() {
-                        return fVersion;
-                    }
-                    // since XMLParseExceptions know nothing about encoding,
-                    // we cannot return anything meaningful in this context.
-                    // We *could* consult the LocatorProxy, but the
-                    // application can do this itself if it wishes to possibly
-                    // be mislead.
-                    public String getEncoding() {
-                        return null;
-                    }
-                };
+            if (ex == null || ex instanceof CharConversionException) {
+                // must be a parser exception; mine it for locator info 
+                // and throw a SAXParseException
+                Locator2Impl locatorImpl = new Locator2Impl();
+                // since XMLParseExceptions know nothing about encoding,
+                // we cannot return anything meaningful in this context.
+                // We *could* consult the LocatorProxy, but the
+                // application can do this itself if it wishes to possibly
+                // be mislead.
+                locatorImpl.setXMLVersion(fVersion);
                 locatorImpl.setPublicId(e.getPublicId());
                 locatorImpl.setSystemId(e.getExpandedSystemId());
                 locatorImpl.setLineNumber(e.getLineNumber());
                 locatorImpl.setColumnNumber(e.getColumnNumber());
-                throw new SAXParseException(e.getMessage(), locatorImpl);
+                throw (ex == null) ? 
+                        new SAXParseException(e.getMessage(), locatorImpl) : 
+                        new SAXParseException(e.getMessage(), locatorImpl, ex);
             }
             if (ex instanceof SAXException) {
                 // why did we create an XMLParseException?
@@ -1203,27 +1201,23 @@ public abstract class AbstractSAXParser
         // wrap XNI exceptions as SAX exceptions
         catch (XMLParseException e) {
             Exception ex = e.getException();
-            if (ex == null) {
-                // must be a parser exception; mine it for locator info and throw
-                // a SAXParseException
-                LocatorImpl locatorImpl = new LocatorImpl() {
-                    public String getXMLVersion() {
-                        return fVersion;
-                    }
-                    // since XMLParseExceptions know nothing about encoding,
-                    // we cannot return anything meaningful in this context.
-                    // We *could* consult the LocatorProxy, but the
-                    // application can do this itself if it wishes to possibly
-                    // be mislead.
-                    public String getEncoding() {
-                        return null;
-                    }
-                };
+            if (ex == null || ex instanceof CharConversionException) {
+                // must be a parser exception; mine it for locator info 
+                // and throw a SAXParseException
+                Locator2Impl locatorImpl = new Locator2Impl();
+                // since XMLParseExceptions know nothing about encoding,
+                // we cannot return anything meaningful in this context.
+                // We *could* consult the LocatorProxy, but the
+                // application can do this itself if it wishes to possibly
+                // be mislead.
+                locatorImpl.setXMLVersion(fVersion);
                 locatorImpl.setPublicId(e.getPublicId());
                 locatorImpl.setSystemId(e.getExpandedSystemId());
                 locatorImpl.setLineNumber(e.getLineNumber());
                 locatorImpl.setColumnNumber(e.getColumnNumber());
-                throw new SAXParseException(e.getMessage(), locatorImpl);
+                throw (ex == null) ? 
+                        new SAXParseException(e.getMessage(), locatorImpl) : 
+                        new SAXParseException(e.getMessage(), locatorImpl, ex);
             }
             if (ex instanceof SAXException) {
                 // why did we create an XMLParseException?
@@ -2198,7 +2192,7 @@ public abstract class AbstractSAXParser
     // Classes
     //
 
-    protected class LocatorProxy
+    protected static final class LocatorProxy
         implements Locator2 {
 
         //
@@ -2303,8 +2297,8 @@ public abstract class AbstractSAXParser
         }
 
         public String getType(String uri, String localName) {
-            return uri.equals("") ? fAttributes.getType(null, localName) :
-                                    fAttributes.getType(uri, localName);
+            return uri.length() == 0 ? fAttributes.getType(null, localName) :
+                                       fAttributes.getType(uri, localName);
         }
 
         public String getValue(int i) {
@@ -2316,8 +2310,8 @@ public abstract class AbstractSAXParser
         }
 
         public String getValue(String uri, String localName) {
-            return uri.equals("") ? fAttributes.getValue(null, localName) :
-                                    fAttributes.getValue(uri, localName);
+            return uri.length() == 0 ? fAttributes.getValue(null, localName) :
+                                       fAttributes.getValue(uri, localName);
         }
 
         public int getIndex(String qName) {
@@ -2325,8 +2319,8 @@ public abstract class AbstractSAXParser
         }
 
         public int getIndex(String uri, String localPart) {
-            return uri.equals("") ? fAttributes.getIndex(null, localPart) :
-                                    fAttributes.getIndex(uri, localPart);
+            return uri.length() == 0 ? fAttributes.getIndex(null, localPart) :
+                                       fAttributes.getIndex(uri, localPart);
         }
         
         // Attributes2 methods
