@@ -18,10 +18,10 @@
 package org.apache.xerces.parsers;
 
 import java.io.StringReader;
+import java.util.ArrayList;
 import java.util.Locale;
 import java.util.Stack;
 import java.util.StringTokenizer;
-import java.util.Vector;
 
 import org.apache.xerces.dom.DOMErrorImpl;
 import org.apache.xerces.dom.DOMMessageFormatter;
@@ -58,13 +58,14 @@ import org.w3c.dom.DOMErrorHandler;
 import org.w3c.dom.DOMException;
 import org.w3c.dom.DOMStringList;
 import org.w3c.dom.Document;
+import org.w3c.dom.Element;
 import org.w3c.dom.Node;
 import org.w3c.dom.ls.LSException;
 import org.w3c.dom.ls.LSInput;
 import org.w3c.dom.ls.LSParser;
 import org.w3c.dom.ls.LSParserFilter;
 import org.w3c.dom.ls.LSResourceResolver;
-import org.xml.sax.SAXException;
+import org.w3c.dom.traversal.NodeFilter;
 
 /**
  * This is Xerces DOM Builder class. It uses the abstract DOM
@@ -74,7 +75,7 @@ import org.xml.sax.SAXException;
  * @author Pavani Mukthipudi, Sun Microsystems Inc.
  * @author Elena Litani, IBM
  * @author Rahul Srivastava, Sun Microsystems Inc.
- * @version $Id: DOMParserImpl.java 466933 2006-10-23 04:52:32Z mrglavas $
+ * @version $Id: DOMParserImpl.java 784915 2009-06-15 18:54:11Z knoaman $
  */
 public class DOMParserImpl
     extends AbstractDOMParser implements LSParser, DOMConfiguration {
@@ -113,6 +114,14 @@ public class DOMParserImpl
     protected static final String HONOUR_ALL_SCHEMALOCATIONS = 
         Constants.XERCES_FEATURE_PREFIX + Constants.HONOUR_ALL_SCHEMALOCATIONS_FEATURE;
 
+    /** Feature identifier: namespace growth */
+    protected static final String NAMESPACE_GROWTH = 
+        Constants.XERCES_FEATURE_PREFIX + Constants.NAMESPACE_GROWTH_FEATURE;
+
+    /** Feature identifier: tolerate duplicates */
+    protected static final String TOLERATE_DUPLICATES = 
+        Constants.XERCES_FEATURE_PREFIX + Constants.TOLERATE_DUPLICATES_FEATURE;
+
     // internal properties
     protected static final String SYMBOL_TABLE =
         Constants.XERCES_PROPERTY_PREFIX + Constants.SYMBOL_TABLE_PROPERTY;
@@ -126,24 +135,24 @@ public class DOMParserImpl
 
     /** Include namespace declaration attributes in the document. **/
     protected boolean fNamespaceDeclarations = true;
-    
+
     // REVISIT: this value should be null by default and should be set during creation of
     //          LSParser
     protected String fSchemaType = null;
 
     protected boolean fBusy = false;
-    
+
     private boolean abortNow = false;
-    
+
     private Thread currentThread;
 
     protected final static boolean DEBUG = false;
 
-    private Vector fSchemaLocations = new Vector ();
     private String fSchemaLocation = null;
-	private DOMStringList fRecognizedParameters;
-    
-    private AbortHandler abortHandler = new AbortHandler();
+    private DOMStringList fRecognizedParameters;
+
+    private boolean fNullFilterInUse = false;
+    private AbortHandler abortHandler = null;
 
     //
     // Constructors
@@ -282,18 +291,21 @@ public class DOMParserImpl
      * @throws SAXException Thrown on initialization error.
      */
     public void reset () {
-        super.reset ();
+        super.reset();
         
         // get state of namespace-declarations parameter.
         fNamespaceDeclarations = 
             fConfiguration.getFeature(Constants.DOM_NAMESPACE_DECLARATIONS);
                 
         // DOM Filter
-        if (fSkippedElemStack!=null) {
-            fSkippedElemStack.removeAllElements ();
+        if (fNullFilterInUse) {
+            fDOMFilter = null;
+            fNullFilterInUse = false;
         }
-        fSchemaLocations.clear ();
-        fRejectedElement.clear ();
+        if (fSkippedElemStack != null) {
+            fSkippedElemStack.removeAllElements();
+        }
+        fRejectedElementDepth = 0;
         fFilterReject = false;
         fSchemaType = null;
 
@@ -307,33 +319,42 @@ public class DOMParserImpl
         return this;
     }
 
-
     /**
-     *  When the application provides a filter, the parser will call out to
-     * the filter at the completion of the construction of each
-     * <code>Element</code> node. The filter implementation can choose to
-     * remove the element from the document being constructed (unless the
-     * element is the document element) or to terminate the parse early. If
-     * the document is being validated when it's loaded the validation
-     * happens before the filter is called.
+     * When a filter is provided, the implementation will call out to the 
+     * filter as it is constructing the DOM tree structure. The filter can 
+     * choose to remove elements from the document being constructed, or to 
+     * terminate the parsing early. 
+     * <br> The filter is invoked after the operations requested by the 
+     * <code>DOMConfiguration</code> parameters have been applied. For 
+     * example, if "<a href='http://www.w3.org/TR/DOM-Level-3-Core/core.html#parameter-validate'>
+     * validate</a>" is set to <code>true</code>, the validation is done before invoking the 
+     * filter. 
      */
     public LSParserFilter getFilter () {
-        return fDOMFilter;
+        return !fNullFilterInUse ? fDOMFilter : null;
     }
 
     /**
-     *  When the application provides a filter, the parser will call out to
-     * the filter at the completion of the construction of each
-     * <code>Element</code> node. The filter implementation can choose to
-     * remove the element from the document being constructed (unless the
-     * element is the document element) or to terminate the parse early. If
-     * the document is being validated when it's loaded the validation
-     * happens before the filter is called.
+     * When a filter is provided, the implementation will call out to the 
+     * filter as it is constructing the DOM tree structure. The filter can 
+     * choose to remove elements from the document being constructed, or to 
+     * terminate the parsing early. 
+     * <br> The filter is invoked after the operations requested by the 
+     * <code>DOMConfiguration</code> parameters have been applied. For 
+     * example, if "<a href='http://www.w3.org/TR/DOM-Level-3-Core/core.html#parameter-validate'>
+     * validate</a>" is set to <code>true</code>, the validation is done before invoking the 
+     * filter. 
      */
     public void setFilter (LSParserFilter filter) {
-        fDOMFilter = filter;
+        if (fBusy && filter == null && fDOMFilter != null) {
+            fNullFilterInUse = true;
+            fDOMFilter = NullLSParserFilter.INSTANCE;
+        }
+        else {
+            fDOMFilter = filter;
+        }
         if (fSkippedElemStack == null) {
-            fSkippedElemStack = new Stack ();
+            fSkippedElemStack = new Stack();
         }
     }
 
@@ -363,12 +384,7 @@ public class DOMParserImpl
                 || name.equalsIgnoreCase (Constants.DOM_CHECK_CHAR_NORMALIZATION)
                 || name.equalsIgnoreCase (Constants.DOM_CANONICAL_FORM)) {
                     if (state) { // true is not supported
-                        String msg =
-                        DOMMessageFormatter.formatMessage (
-                        DOMMessageFormatter.DOM_DOMAIN,
-                        "FEATURE_NOT_SUPPORTED",
-                        new Object[] { name });
-                        throw new DOMException (DOMException.NOT_SUPPORTED_ERR, msg);
+                        throw newFeatureNotSupportedError(name);
                     }
                     // setting those features to false is no-op
                 }
@@ -402,12 +418,7 @@ public class DOMParserImpl
                 else if (name.equalsIgnoreCase (Constants.DOM_WELLFORMED)
                 || name.equalsIgnoreCase (Constants.DOM_IGNORE_UNKNOWN_CHARACTER_DENORMALIZATIONS)) {
                     if (!state) { // false is not supported
-                        String msg =
-                        DOMMessageFormatter.formatMessage (
-                        DOMMessageFormatter.DOM_DOMAIN,
-                        "FEATURE_NOT_SUPPORTED",
-                        new Object[] { name });
-                        throw new DOMException (DOMException.NOT_SUPPORTED_ERR, msg);
+                        throw newFeatureNotSupportedError(name);
                     }
                     // setting these features to true is no-op
                     // REVISIT: implement "namespace-declaration" feature
@@ -448,6 +459,12 @@ public class DOMParserImpl
                     if (name.equalsIgnoreCase(HONOUR_ALL_SCHEMALOCATIONS)) {
                         normalizedName = HONOUR_ALL_SCHEMALOCATIONS;
                     }
+                    else if (name.equals(NAMESPACE_GROWTH)) {
+                        normalizedName = NAMESPACE_GROWTH;
+                    }
+                    else if (name.equals(TOLERATE_DUPLICATES)) {
+                        normalizedName = TOLERATE_DUPLICATES;
+                    }
                     else {
                         normalizedName = name.toLowerCase(Locale.ENGLISH);
                     }
@@ -456,12 +473,7 @@ public class DOMParserImpl
 
             }
             catch (XMLConfigurationException e) {
-                String msg =
-                DOMMessageFormatter.formatMessage (
-                DOMMessageFormatter.DOM_DOMAIN,
-                "FEATURE_NOT_FOUND",
-                new Object[] { name });
-                throw new DOMException (DOMException.NOT_FOUND_ERR, msg);
+                throw newFeatureNotFoundError(name);
             }
         }
         else { // set properties
@@ -474,13 +486,7 @@ public class DOMParserImpl
                     catch (XMLConfigurationException e) {}
                 }
                 else {
-                    // REVISIT: type mismatch
-                    String msg =
-                    DOMMessageFormatter.formatMessage (
-                    DOMMessageFormatter.DOM_DOMAIN,
-                    "TYPE_MISMATCH_ERR",
-                    new Object[] { name });
-                    throw new DOMException (DOMException.TYPE_MISMATCH_ERR, msg);
+                    throw newTypeMismatchError(name);
                 }
 
             }
@@ -492,13 +498,7 @@ public class DOMParserImpl
                     catch (XMLConfigurationException e) {}
                 }
                 else {
-                    // REVISIT: type mismatch
-                    String msg =
-                    DOMMessageFormatter.formatMessage (
-                    DOMMessageFormatter.DOM_DOMAIN,
-                    "TYPE_MISMATCH_ERR",
-                    new Object[] { name });
-                    throw new DOMException (DOMException.TYPE_MISMATCH_ERR, msg);
+                    throw newTypeMismatchError(name);
                 }
 
             }
@@ -516,15 +516,15 @@ public class DOMParserImpl
                             // map DOM schema-location to JAXP schemaSource property
                             // tokenize location string
                             StringTokenizer t = new StringTokenizer (fSchemaLocation, " \n\t\r");
-                            if (t.hasMoreTokens ()){
-                                fSchemaLocations.clear ();
-                                fSchemaLocations.add (t.nextToken ());
-                                while (t.hasMoreTokens ()) {
-                                    fSchemaLocations.add (t.nextToken ());
+                            if (t.hasMoreTokens()) {
+                                ArrayList locations = new ArrayList();
+                                locations.add (t.nextToken());
+                                while (t.hasMoreTokens()) {
+                                    locations.add (t.nextToken());
                                 }
                                 fConfiguration.setProperty (
                                 Constants.JAXP_PROPERTY_PREFIX + Constants.SCHEMA_SOURCE,
-                                fSchemaLocations.toArray ());
+                                locations.toArray ());
                             }
                             else {
                                 fConfiguration.setProperty (
@@ -536,13 +536,7 @@ public class DOMParserImpl
                     catch (XMLConfigurationException e) {}
                 }
                 else {
-                    // REVISIT: type mismatch
-                    String msg =
-                    DOMMessageFormatter.formatMessage (
-                    DOMMessageFormatter.DOM_DOMAIN,
-                    "TYPE_MISMATCH_ERR",
-                    new Object[] { name });
-                    throw new DOMException (DOMException.TYPE_MISMATCH_ERR, msg);
+                    throw newTypeMismatchError(name);
                 }
 
             }
@@ -583,12 +577,7 @@ public class DOMParserImpl
                     catch (XMLConfigurationException e) {}
                 }
                 else {
-                    String msg =
-                    DOMMessageFormatter.formatMessage (
-                    DOMMessageFormatter.DOM_DOMAIN,
-                    "TYPE_MISMATCH_ERR",
-                    new Object[] { name });
-                    throw new DOMException (DOMException.TYPE_MISMATCH_ERR, msg);
+                    throw newTypeMismatchError(name);
                 }
 
             }
@@ -611,24 +600,20 @@ public class DOMParserImpl
                     if (name.equalsIgnoreCase(HONOUR_ALL_SCHEMALOCATIONS)) {
                         normalizedName = HONOUR_ALL_SCHEMALOCATIONS;
                     }
+                    else if (name.equals(NAMESPACE_GROWTH)) {
+                        normalizedName = NAMESPACE_GROWTH;
+                    }
+                    else if (name.equals(TOLERATE_DUPLICATES)) {
+                        normalizedName = TOLERATE_DUPLICATES;
+                    }
                     fConfiguration.getFeature(normalizedName);
-                    String msg =
-                        DOMMessageFormatter.formatMessage (
-                                DOMMessageFormatter.DOM_DOMAIN,
-                                "TYPE_MISMATCH_ERR",
-                                new Object[] { name });
-                    throw new DOMException (DOMException.TYPE_MISMATCH_ERR, msg);
+                    throw newTypeMismatchError(name);
                     
                 }
                 catch (XMLConfigurationException e) {}
                 
                 // Parameter is not recognized
-                String msg =
-                    DOMMessageFormatter.formatMessage (
-                            DOMMessageFormatter.DOM_DOMAIN,
-                            "FEATURE_NOT_FOUND",
-                            new Object[] { name });
-                throw new DOMException (DOMException.NOT_FOUND_ERR, msg);
+                throw newFeatureNotFoundError(name);
             }
         }
     }
@@ -751,6 +736,12 @@ public class DOMParserImpl
             if (name.equalsIgnoreCase(HONOUR_ALL_SCHEMALOCATIONS)) {
                 normalizedName = HONOUR_ALL_SCHEMALOCATIONS;
             }
+            else if (name.equals(NAMESPACE_GROWTH)) {
+                normalizedName = NAMESPACE_GROWTH;
+            }
+            else if (name.equals(TOLERATE_DUPLICATES)) {
+                normalizedName = TOLERATE_DUPLICATES;
+            }
             else {
                 normalizedName = name.toLowerCase(Locale.ENGLISH);
             }
@@ -766,12 +757,7 @@ public class DOMParserImpl
             }
             catch (XMLConfigurationException e) {}
             
-            String msg =
-                DOMMessageFormatter.formatMessage (
-                        DOMMessageFormatter.DOM_DOMAIN,
-                        "FEATURE_NOT_FOUND",
-                        new Object[] { name });
-            throw new DOMException (DOMException.NOT_FOUND_ERR, msg);
+            throw newFeatureNotFoundError(name);
         }
     }
 
@@ -818,6 +804,12 @@ public class DOMParserImpl
                 if (name.equalsIgnoreCase(HONOUR_ALL_SCHEMALOCATIONS)) {
                     normalizedName = HONOUR_ALL_SCHEMALOCATIONS;
                 }
+                else if (name.equalsIgnoreCase(NAMESPACE_GROWTH)) {
+                    normalizedName = NAMESPACE_GROWTH;
+                }
+                else if (name.equalsIgnoreCase(TOLERATE_DUPLICATES)) {
+                    normalizedName = TOLERATE_DUPLICATES;
+                }
                 else {
                     normalizedName = name.toLowerCase(Locale.ENGLISH);
                 }
@@ -879,7 +871,7 @@ public class DOMParserImpl
      */
     public DOMStringList getParameterNames () {
         if (fRecognizedParameters == null){
-            Vector parameters = new Vector();
+            ArrayList parameters = new ArrayList();
             
             // REVISIT: add Xerces recognized properties/features
             parameters.add(Constants.DOM_NAMESPACES);
@@ -928,10 +920,7 @@ public class DOMParserImpl
         //If DOMParser insstance is already busy parsing another document when this
         // method is called, then raise INVALID_STATE_ERR according to DOM L3 LS spec
         if ( fBusy ) {
-            String msg = DOMMessageFormatter.formatMessage (
-            DOMMessageFormatter.DOM_DOMAIN,
-            "INVALID_STATE_ERR",null);
-            throw new DOMException ( DOMException.INVALID_STATE_ERR,msg);
+            throw newInvalidStateError();
         }
 
         XMLInputSource source = new XMLInputSource (null, uri, null);
@@ -957,7 +946,7 @@ public class DOMParserImpl
             }
             // Consume this exception if the user
             // issued an interrupt or an abort.
-            if (e != abort) {
+            if (e != Abort.INSTANCE) {
                 if (!(e instanceof XMLParseException) && fErrorHandler != null) {
                     DOMErrorImpl error = new DOMErrorImpl ();
                     error.fException = e;
@@ -986,10 +975,7 @@ public class DOMParserImpl
         // need to wrap the LSInput with an XMLInputSource
         XMLInputSource xmlInputSource = dom2xmlInputSource (is);
         if ( fBusy ) {
-            String msg = DOMMessageFormatter.formatMessage (
-            DOMMessageFormatter.DOM_DOMAIN,
-            "INVALID_STATE_ERR",null);
-            throw new DOMException ( DOMException.INVALID_STATE_ERR,msg);
+            throw newInvalidStateError();
         }
 
         try {
@@ -1014,7 +1000,7 @@ public class DOMParserImpl
             }
             // Consume this exception if the user
             // issued an interrupt or an abort.
-            if (e != abort) {
+            if (e != Abort.INSTANCE) {
                 if (!(e instanceof XMLParseException) && fErrorHandler != null) {
                    DOMErrorImpl error = new DOMErrorImpl ();
                    error.fException = e;
@@ -1134,18 +1120,19 @@ public class DOMParserImpl
      */
     public void abort () {
         // If parse operation is in progress then reset it
-        if ( fBusy ) {
+        if (fBusy) {
             fBusy = false;
-            if(currentThread != null) {
+            if (currentThread != null) {
                 abortNow = true;
-                
+                if (abortHandler == null) {
+                    abortHandler = new AbortHandler();
+                }
                 fConfiguration.setDocumentHandler(abortHandler);
                 fConfiguration.setDTDHandler(abortHandler);
                 fConfiguration.setDTDContentModelHandler(abortHandler);
-                
-                if(currentThread == Thread.currentThread())
-                    throw abort;
-                
+                if (currentThread == Thread.currentThread()) {
+                    throw Abort.INSTANCE;
+                }
                 currentThread.interrupt();
             }               
         }
@@ -1178,74 +1165,88 @@ public class DOMParserImpl
         super.startElement(element, attributes, augs);
     }
     
-    private class AbortHandler implements XMLDocumentHandler, XMLDTDHandler, XMLDTDContentModelHandler  {
+    static final class NullLSParserFilter implements LSParserFilter {
+        static final NullLSParserFilter INSTANCE = new NullLSParserFilter();
+        private NullLSParserFilter() {}
+        public short acceptNode(Node nodeArg) {
+            return LSParserFilter.FILTER_ACCEPT;
+        }
+        public int getWhatToShow() {
+            return NodeFilter.SHOW_ALL;
+        }
+        public short startElement(Element elementArg) {
+            return LSParserFilter.FILTER_ACCEPT;
+        }
+    }
+    
+    private static final class AbortHandler implements XMLDocumentHandler, XMLDTDHandler, XMLDTDContentModelHandler {
 
         private XMLDocumentSource documentSource;
         private XMLDTDContentModelSource dtdContentSource;
         private XMLDTDSource dtdSource;
 
         public void startDocument(XMLLocator locator, String encoding, NamespaceContext namespaceContext, Augmentations augs) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void xmlDecl(String version, String encoding, String standalone, Augmentations augs) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void doctypeDecl(String rootElement, String publicId, String systemId, Augmentations augs) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void comment(XMLString text, Augmentations augs) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void processingInstruction(String target, XMLString data, Augmentations augs) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void startElement(QName element, XMLAttributes attributes, Augmentations augs) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void emptyElement(QName element, XMLAttributes attributes, Augmentations augs) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void startGeneralEntity(String name, XMLResourceIdentifier identifier, String encoding, Augmentations augs) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void textDecl(String version, String encoding, Augmentations augs) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void endGeneralEntity(String name, Augmentations augs) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void characters(XMLString text, Augmentations augs) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void ignorableWhitespace(XMLString text, Augmentations augs) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void endElement(QName element, Augmentations augs) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void startCDATA(Augmentations augs) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void endCDATA(Augmentations augs) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void endDocument(Augmentations augs) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void setDocumentSource(XMLDocumentSource source) {
@@ -1257,71 +1258,71 @@ public class DOMParserImpl
         }
 
         public void startDTD(XMLLocator locator, Augmentations augmentations) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void startParameterEntity(String name, XMLResourceIdentifier identifier, String encoding, Augmentations augmentations) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void endParameterEntity(String name, Augmentations augmentations) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void startExternalSubset(XMLResourceIdentifier identifier, Augmentations augmentations) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void endExternalSubset(Augmentations augmentations) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void elementDecl(String name, String contentModel, Augmentations augmentations) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void startAttlist(String elementName, Augmentations augmentations) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void attributeDecl(String elementName, String attributeName, String type, String[] enumeration, String defaultType, XMLString defaultValue, XMLString nonNormalizedDefaultValue, Augmentations augmentations) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void endAttlist(Augmentations augmentations) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void internalEntityDecl(String name, XMLString text, XMLString nonNormalizedText, Augmentations augmentations) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void externalEntityDecl(String name, XMLResourceIdentifier identifier, Augmentations augmentations) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void unparsedEntityDecl(String name, XMLResourceIdentifier identifier, String notation, Augmentations augmentations) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void notationDecl(String name, XMLResourceIdentifier identifier, Augmentations augmentations) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void startConditional(short type, Augmentations augmentations) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void ignoredCharacters(XMLString text, Augmentations augmentations) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void endConditional(Augmentations augmentations) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void endDTD(Augmentations augmentations) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void setDTDSource(XMLDTDSource source) {
@@ -1333,43 +1334,43 @@ public class DOMParserImpl
         }
 
         public void startContentModel(String elementName, Augmentations augmentations) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void any(Augmentations augmentations) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void empty(Augmentations augmentations) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void startGroup(Augmentations augmentations) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void pcdata(Augmentations augmentations) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void element(String elementName, Augmentations augmentations) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void separator(short separator, Augmentations augmentations) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void occurrence(short occurrence, Augmentations augmentations) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void endGroup(Augmentations augmentations) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void endContentModel(Augmentations augmentations) throws XNIException {
-            throw abort;
+            throw Abort.INSTANCE;
         }
 
         public void setDTDContentModelSource(XMLDTDContentModelSource source) {
@@ -1378,8 +1379,42 @@ public class DOMParserImpl
 
         public XMLDTDContentModelSource getDTDContentModelSource() {
             return dtdContentSource;
-        }
-        
+        }  
+    }
+    
+    private static DOMException newInvalidStateError() {
+        String msg = 
+            DOMMessageFormatter.formatMessage (
+                    DOMMessageFormatter.DOM_DOMAIN,
+                    "INVALID_STATE_ERR", null);
+        throw new DOMException ( DOMException.INVALID_STATE_ERR, msg);
+    }
+    
+    private static DOMException newFeatureNotSupportedError(String name) {
+        String msg =
+            DOMMessageFormatter.formatMessage (
+                    DOMMessageFormatter.DOM_DOMAIN,
+                    "FEATURE_NOT_SUPPORTED",
+                    new Object[] { name });
+        return new DOMException (DOMException.NOT_SUPPORTED_ERR, msg);
+    }
+    
+    private static DOMException newFeatureNotFoundError(String name) {
+        String msg =
+            DOMMessageFormatter.formatMessage (
+                    DOMMessageFormatter.DOM_DOMAIN,
+                    "FEATURE_NOT_FOUND",
+                    new Object[] { name });
+        return new DOMException (DOMException.NOT_FOUND_ERR, msg);
+    }
+    
+    private static DOMException newTypeMismatchError(String name) {
+        String msg =
+            DOMMessageFormatter.formatMessage (
+                    DOMMessageFormatter.DOM_DOMAIN,
+                    "TYPE_MISMATCH_ERR",
+                    new Object[] { name });
+        return new DOMException (DOMException.TYPE_MISMATCH_ERR, msg);
     }
 	
 } // class DOMParserImpl
