@@ -1,9 +1,10 @@
 /*
- * Copyright 2000-2002,2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -37,7 +38,7 @@ import org.apache.xerces.xni.XMLAttributes;
  * @author Elena Litani, IBM
  * @author Michael Glavassevich, IBM
  *
- * @version $Id: XMLAttributesImpl.java 319864 2004-03-25 04:03:23Z mrglavas $
+ * @version $Id: XMLAttributesImpl.java 937816 2010-04-25 16:22:27Z mrglavas $
  */
 public class XMLAttributesImpl
     implements XMLAttributes {
@@ -174,7 +175,7 @@ public class XMLAttributesImpl
 
         int index;
         if (fLength < SIZE_LIMIT) {
-            index = name.uri != null && !name.uri.equals("") 
+            index = name.uri != null && name.uri.length() != 0
                 ? getIndexFast(name.uri, name.localpart)
                 : getIndexFast(name.rawname);
 
@@ -735,26 +736,6 @@ public class XMLAttributesImpl
     } // getURI(int,QName)
 
     // Implementation methods
-    public void setSchemaId(int attrIndex, boolean schemaId) {
-        fAttributes[attrIndex].schemaId = schemaId;
-    }
-    public boolean getSchemaId(int index) {
-        if (index < 0 || index >= fLength) {
-            return false;
-        }
-        return fAttributes[index].schemaId;
-    }
-    public boolean getSchemaId(String qname) {
-        int index = getIndex(qname);
-        return index != -1 ? fAttributes[index].schemaId : false; 
-    } // getType(String):String
-    public boolean getSchemaId(String uri, String localName) {
-        if (!fNamespaces) {
-            return false;
-        }
-        int index = getIndex(uri, localName);
-        return index != -1 ? fAttributes[index].schemaId : false;
-    } // getType(String,String):String
     
     /**
      * Look up the index of an attribute by XML 1.0 qualified name.
@@ -1046,7 +1027,7 @@ public class XMLAttributesImpl
         // basic info
 
         /** Name. */
-        public QName name = new QName();
+        public final QName name = new QName();
 
         /** Type. */
         public String type;
@@ -1059,9 +1040,6 @@ public class XMLAttributesImpl
 
         /** Specified. */
         public boolean specified;
-
-        /** Schema ID type. */
-        public boolean schemaId;
         
         /** 
          * Augmentations information for this attribute.
