@@ -14,6 +14,7 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+
 package org.apache.xerces.xpointer;
 
 import org.apache.xerces.impl.Constants;
@@ -34,10 +35,9 @@ import org.apache.xerces.xs.XSTypeDefinition;
  * in document order that has a matching NCName as an identifier.
  * </p>
  *
- * @version $Id: ShortHandPointer.java 447248 2006-09-18 05:25:21Z mrglavas $
- *
+ * @version $Id: ShortHandPointer.java 603808 2007-12-13 03:44:48Z mrglavas $
  */
-class ShortHandPointer implements XPointerPart {
+final class ShortHandPointer implements XPointerPart {
     
     // The name of the ShortHand pointer
     private String fShortHandPointer;
