@@ -33,7 +33,8 @@ package org.apache.xml.serialize;
 import java.io.IOException;
 import java.io.OutputStream;
 import java.io.Writer;
-import java.util.Enumeration;
+import java.util.Iterator;
+import java.util.Map;
 
 import org.apache.xerces.dom.DOMMessageFormatter;
 import org.apache.xerces.util.NamespaceSupport;
@@ -43,6 +44,7 @@ import org.apache.xerces.util.XMLSymbols;
 import org.apache.xerces.xni.NamespaceContext;
 import org.w3c.dom.Attr;
 import org.w3c.dom.DOMError;
+import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 import org.w3c.dom.NamedNodeMap;
 import org.w3c.dom.Node;
@@ -86,7 +88,7 @@ import org.xml.sax.helpers.AttributesImpl;
  * @author <a href="mailto:arkin@intalio.com">Assaf Arkin</a>
  * @author <a href="mailto:rahul.srivastava@sun.com">Rahul Srivastava</a>
  * @author Elena Litani IBM
- * @version $Revision: 476047 $ $Date: 2006-11-16 23:27:45 -0500 (Thu, 16 Nov 2006) $
+ * @version $Revision: 704573 $ $Date: 2008-10-14 12:11:22 -0400 (Tue, 14 Oct 2008) $
  * @see Serializer
  */
 public class XMLSerializer
@@ -335,18 +337,18 @@ extends BaseMarkupSerializer {
             }
 
             if (_prefixes != null) {
-                Enumeration keys;
-
-                keys = _prefixes.keys();
-                while (keys.hasMoreElements()) {
+                Iterator entries = _prefixes.entrySet().iterator();
+                while (entries.hasNext()) {
                     _printer.printSpace();
-                    value = (String) keys.nextElement();
-                    name = (String) _prefixes.get( value );
+                    Map.Entry entry = (Map.Entry) entries.next();
+                    value = (String) entry.getKey();
+                    name = (String) entry.getValue();
                     if (name.length() == 0) {
                         _printer.printText( "xmlns=\"" );
                         printEscaped( value );
                         _printer.printText( '"' );
-                    } else {
+                    } 
+                    else {
                         _printer.printText( "xmlns:" );
                         _printer.printText( name );
                         _printer.printText( "=\"" );
